#ifndef INVERSEDCT2D_INC
#define INVERSEDCT2D_INC

#include "ipCore.h"
#include "ipDCT2D.h"

namespace Torch {

/** This class is designed to reconstruct an image given DCT coefficients

    \begin{verbatim}

      seq_in->frames[0]        +--------------+
	       +----+          |              |
	       |    |   ---->  |   ipDCT2D    | ----> seq_out->frames[0] (DCT coefficients)
	       |    |          |              |
	       +----+          +--------------+
             (8x8 image)
	 
    \end{verbatim}
    
    @author Sebastien Marcel (marcel@idiap.ch)
    
    @version 2.0
    \Date
    @since 2.0

    @see ipDCT2D
*/
class ipDCT2Dinverse : public ipCore
{
   	// DCT2D 
	ipDCT2D *dct;

	// number of DCT coefficients
	int dct_dim;

	// size of the image
	int block_size;
	
public:	
	/** constructor
	
	    @param dctdim_ is the number of DCT coefficients to use
	    @param block_size_ is the size of the output image
	*/
	ipDCT2Dinverse(int dctdim_, int block_size_);

	///
    	virtual ~ipDCT2Dinverse();
	
	///
    	virtual void process(Sequence *seq_in);
};

}

#endif
