#include "ipOneBlob.h"
#include "fastMatrix.h"

namespace Torch {
  
ipOneBlob::ipOneBlob(int width_, int height_, const char *colorcube_filename, const char *coding_) : ipCore(coding_)
{    
    width = width_;
    height = height_;
    if(strcmp(coding_, "rgb") == 0)
      {
	ipcolorcube = new(allocator) ipColorCube(width, height, "rgb");
	ipcolorcube->load(colorcube_filename);	
	
	seq_out = new(allocator) Sequence(1,5);
	for(int i = 0; i < 5; i++)
	  seq_out->frames[0][i] = 0.0;
      }
    else
      error("Only RGB-format permited");
}
  
void ipOneBlob::process(Sequence *seq_in)
{
    ipcolorcube->process(seq_in);
    
    int mu_x1 = 0;
    int mu_y1 = 0;
    real lambda1 = 0.;
    real lambda2 = 0.;
    real vecteur1[2];
    real vecteur2[2];
    real angle = 0.;
    real excentricity = 0.;
    
    int count = 0;

    // Pour les calculs matriciels
    fastMatrix mat(2);
    
    real *xy = (real *)allocator->alloc(sizeof(real)*2);

    real **mat_transit;
    mat_transit = (real **)allocator->alloc(sizeof(real *)*2);
    for(int i=0; i<2; i++)
      mat_transit[i] = (real *)allocator->alloc(sizeof(real)*2);

    real **sigma;
    sigma = (real **)allocator->alloc(sizeof(real *)*2);
    for(int i=0; i<2; i++)
      sigma[i] = (real *)allocator->alloc(sizeof(real)*2);
    
    int size = width*height;

    // Debut du processing    
    for(int i = 0 ; i < size ; i++)
      {
	/** Calcul de la moyenne **/
	if(ipcolorcube->binaryimage[i] == 1)
	  {
	    mu_x1 += div(i,width).rem;
	    mu_y1 += div(i,width).quot;
	    count++;
	  }
      }
    
    // pour le premier colorcube...
    if(count < 1)
      {
	//message("pas de pixel de couleur!");
	for(int i=0; i<5; i++)
	  seq_out->frames[0][i] = -1.0;
      }
    else
      {
	mu_x1 = mu_x1 / count;
	mu_y1 = mu_y1 / count;
	seq_out->frames[0][0] = mu_x1;
	seq_out->frames[0][1] = mu_y1;
		
	/*** Calcul de la matrice de variance-covariance ***/
	mat.initVector(xy);
	mat.initMatrix(mat_transit);
	mat.initMatrix(sigma);

	count = 0;
	for (int k = 0; k < size; k++)
	  {
	    if(ipcolorcube->binaryimage[k] == 1)
	      {	
		count = count + 1;
		xy[0] = (double) div(k,width).rem - mu_x1;    //correspond  x
		xy[1] = (double) div(k,width).quot - mu_y1;   //correspond  y
		// X_i . X_i^T
		mat.vectorialProduct(mat_transit, xy, xy);
		// \sigma X_i . X_i^T
		mat.addMatrix(sigma, mat_transit);
	      }	      
	  }
	for (int l = 0; l < 2; l++)
	  for (int c = 0; c < 2; c++)
	    {
	      sigma[l][c] = sigma[l][c] / count;
	    }
	//printf("sigma:\n");
	//mat.printMatrix(sigma);
	
	for (int i = 0; i < size; i++)
	  {
	    if(ipcolorcube->binaryimage[i] == 1)
	      {		  
		if ((div(i,width).rem < (mu_x1 - 3*sqrt(sigma[0][0]))) || (div(i,width).rem > (mu_x1 + 3*sqrt(sigma[0][0]))))
		  ipcolorcube->binaryimage[i] = 0;
		if ((div(i,width).quot < (mu_y1 - 3*sqrt(sigma[1][1]))) || (div(i,width).quot > (mu_y1 + 3*sqrt(sigma[1][1]))))
		  ipcolorcube->binaryimage[i] = 0;
	      }
	  }
	
	//  Refaire calcul de la matrice de variance covariance...	  
	count = 0;
	for (int k = 0; k < size; k++)
	  {
	    if(ipcolorcube->binaryimage[k] == 1)
	      {	
		count = count + 1;
		xy[0] = (double) div(k,width).rem - mu_x1;    //correspond  x
		xy[1] = (double) div(k,width).quot - mu_y1;   //correspond  y
		
		// X_i . X_i^T
		mat.vectorialProduct(mat_transit, xy, xy);
		
		// \sigma X_i . X_i^T
		mat.addMatrix(sigma, mat_transit);
	      }	      
	  }
	
	for (int l = 0; l < 2; l++)
	  for (int c = 0; c < 2; c++)
	    {
	      sigma[l][c] = sigma[l][c] / count;
	    }
	//printf("sigma:\n");
	//mat.printMatrix(sigma);
	
	//printf("count = %d\n", count);

	double temp_var = sigma[0][0] + sigma[1][1];
	double temp_var_bis = sigma[0][0] - sigma[1][1];	 
	
	//Calcul des valeurs propres...
	lambda1 = (temp_var + sqrt(pow(temp_var_bis, 2) + 4 * pow(sigma[0][1], 2))) / 2;
	lambda2 = (temp_var - sqrt(pow(temp_var_bis, 2) + 4 * pow(sigma[0][1], 2))) / 2;
	//printf("lambda1 = %2.6f\t lambda2 = %2.6f\n", lambda1, lambda2); //c'est OK
	
	double norme_vecteur1 = 0;
	vecteur1[0] = 2 * sigma[0][1];
	vecteur1[1] = 2 * lambda1 - 2 * sigma[0][0];
	
	if (lambda1 > lambda2)
	  {
	    angle = atan(vecteur1[1] / vecteur1[0]);
	    excentricity = sqrt(1-pow(lambda2/lambda1,2));
	  }
	else
	  {
	    angle = atan(-vecteur1[0] / vecteur1[1]);
	    excentricity = sqrt(1-pow(lambda1/lambda2,2));
	  }	
	
	norme_vecteur1 = sqrt(pow(vecteur1[0],2) + pow(vecteur1[1],2));
	vecteur1[0] = vecteur1[0] / norme_vecteur1;
	vecteur1[1] = vecteur1[1] / norme_vecteur1;
	//printf("vecteur[1] = %2.6f\t vecteur[2] = %2.6f\n", vecteur1[0], vecteur1[1]); //c'est OK

	vecteur2[0] = -vecteur1[1];
	vecteur2[1] = vecteur1[0];
	//printf("vecteur[1] = %2.6f\t vecteur[2] = %2.6f\n", vecteur2[0], vecteur2[1]); //c'est OK
	
	seq_out->frames[0][2] = excentricity;
	seq_out->frames[0][3] = angle;
	seq_out->frames[0][4] = M_PI*4*sqrt(lambda1) * sqrt(lambda2);
      }
}  
  
ipOneBlob::~ipOneBlob()
{
}
  
}

