#include "ipScale_yx.h"

namespace Torch {

ipScale_yx::ipScale_yx(int width_in_, int height_in_, char *coding_, int width_out_, int height_out_) : ipCore(coding_)
{
	width_in = width_in_;
        height_in = height_in_;

	width_out = width_out_;
        height_out = height_out_;	
	
   	size_buffer = 3 * width_in * n_planes + width_out * n_planes;
	buffer = (real *)allocator->alloc(size_buffer*sizeof(real));
	memset(buffer, 0, size_buffer*sizeof(real));

	seq_out = new(allocator) Sequence(1, width_out_*height_out_*n_planes);
}

void ipScale_yx::process(Sequence *seq_in)
{
	switch(coding_id)
	{
	case 0:
		YX_scale_gray(width_in, height_in, seq_in->frames[0], width_out, height_out, seq_out->frames[0], buffer);
		break;
	case 1:
		YX_scale_rgb(width_in, height_in, seq_in->frames[0], width_out, height_out, seq_out->frames[0], buffer);
		break;
	case 2:
	case 3:
		message("ipScale_yx::process() not implemented for YUV and float format");
		break;
	}
}

void ipScale_yx::setInputSize(int width, int height)
{
	width_in = width;
        height_in = height;

	size_buffer = 3 * width_in * n_planes + width_out * n_planes;
	allocator->free(buffer);
	buffer = (real *)allocator->alloc(size_buffer*sizeof(real));
	for(int i = 0 ; i < size_buffer ; i++) buffer[i] = 0.0;
}

int ipScale_yx::getWidthOut(void)
{
        return width_out;
}

int ipScale_yx::getHeightOut(void)
{
        return height_out;
}

void ipScale_yx::setOutputSize(int width, int height)
{
        width_out = width;
        height_out = height;

	allocator->free(seq_out);
	seq_out = new(allocator) Sequence(1, width_out*height_out*n_planes);

	size_buffer = 3 * width_in * n_planes + width_out * n_planes;
	allocator->free(buffer);
	buffer = (real *)allocator->alloc(size_buffer*sizeof(real));
	for(int i = 0 ; i < size_buffer ; i++) buffer[i] = 0.0;
}

ipScale_yx::~ipScale_yx()
{
}

int YX_scale_gray(int width_src, int height_src, real *src_, int width_dst, int height_dst, real *dst_, real *buffer)
{
	float *image = (float *) src_;
	float *scale_image = (float *) dst_;

	/*
	for(int i = 0 ; i < width_src * height_src ; i++)
	   	if(isnan(image[i])) print("YX_scale_gray(): isnan in image\n");
	
	for(int i = 0 ; i < 3 * width_src + width_dst ; i++)
	   	if(isnan(buffer[i])) print("YX_scale_gray(): isnan in buffer\n");
	*/	
	
	float
		x_scale,
		x_scale_const,
		x_span,
		y_scale,
		y_scale_const,
		y_span;

	float	
		pixel,
		*scale_scanline,
		*scanline,
		*x_vector,
		*y_vector,
		zero;

	int	
		number_rows, y;

	float max_pixel = 255.0;

	register const float
		*p;

	register int
		i,
		x;

	register float
		*q;

	register float
		*s,
		*t;

	int
		next_column,
		next_row;

	//  
	x_vector = &buffer[0];

	//
	if(height_src != height_dst) scanline = &buffer[width_src];
   	else scanline = x_vector;

	//
	y_vector = &buffer[2*width_src];
	
	//
	scale_scanline = &buffer[3*width_src];

	/*
		Scale image
	*/
	number_rows=0;
	next_row=1;
	y_span=1.0;
        x_scale_const = (float) width_dst/width_src;
	y_scale_const = (float) height_dst/height_src;
	y_scale = y_scale_const;
	
	zero = 0.0;
	
	i=0;
	for (y=0; y < height_dst; y++)
	{
		q = &scale_image[y*width_dst];
		
		if (q == NULL)
			break;

    		if(height_dst == height_src)
      		{
        		/*
          			Read a new scanline.
        		*/
			p = &image[i*width_src];
			i++;
			
        		if (p == NULL)
          			break;
			
        		for (x=0; x < width_src; x++)
        		{
          			x_vector[x] = *p;
          			p++;
        		}
      		}
    		else
      		{
        		/*
          			Scale Y direction.
        		*/
        		while (y_scale < y_span)
        		{
          			if (next_row && (number_rows < height_src))
            			{
              				/*
                				Read a new scanline.
              				*/
					p = &image[i*width_src];
					i++;
				
              				if (p == NULL)
                				break;
				
              				for (x=0; x < width_src; x++)
              				{
                				x_vector[x] = *p;
                				p++;
              				}
              				number_rows++;
            			}
			
          			for (x=0; x < width_src; x++)
            				y_vector[x] += y_scale * x_vector[x];
				
          			y_span-=y_scale;
          			y_scale = y_scale_const;
          			next_row=1;
        		}
		
        		if (next_row && (number_rows < height_src))
          		{
            			/*
              				Read a new scanline.
            			*/
				p = &image[i*width_src];
				i++;
			
            			if (p == NULL)
              				break;
			
            			for (x=0; x < width_src; x++)
            			{
              				x_vector[x] = *p;
              				p++;
            			}
            			number_rows++;
            			next_row=0;
          		}
		
        		s=scanline;
        		for (x=0; x < width_src; x++)
        		{
          			pixel = y_vector[x] + y_span * x_vector[x];

          			*s = pixel > max_pixel ? max_pixel : pixel;
          			s++;

          			y_vector[x] = 0;
        		}
		
        		y_scale-=y_span;
        		if (y_scale <= 0)
          		{
            			y_scale = y_scale_const;
            			next_row=1;
          		}
        		y_span=1.0;
      		}
	
    		if (width_dst == width_src)
      		{
        		/*
          			Transfer scanline to scaled image.
        		*/
        		s=scanline;
        		for (x=0; x < width_dst; x++)
        		{
          			*q = *s;
          			q++;
          			s++;
        		}
      		}
    		else
      		{
        		/*
          			Scale X direction.
        		*/
        		pixel=zero;
        		next_column=0;
        		x_span=1.0;
        		s=scanline;
        		t=scale_scanline;
        		for (x=0; x < width_src; x++)
        		{
          			x_scale = x_scale_const;

          			while (x_scale >= x_span)
          			{
            				if (next_column)
              				{
                				pixel=zero;
                				t++;
              				}
            				pixel += x_span * *s;
            				*t = pixel > max_pixel ? max_pixel : pixel;
            				x_scale-=x_span;
            				x_span=1.0;
            				next_column=1;
          			}
        
				if (x_scale > 0)
          			{
            				if (next_column)
              				{
                				pixel=zero;
                				next_column=0;
                				t++;
              				}
            				pixel += x_scale * *s;
            				x_span-=x_scale;
          			}
        			s++;
      			}
      
			if (x_span > 0)
        		{
          			s--;
          			pixel += x_span * *s;
        		}
      
			if (!next_column && ((t-scale_scanline) < width_dst))
          			*t = pixel > max_pixel ? max_pixel : pixel;
      
			/*
        			Transfer scanline to scaled image.
      			*/
      			t=scale_scanline;
      			for (x=0; x < width_dst; x++)
      			{
        			*q = *t;
        			q++;
        			t++;
      			}
    		}
	}

	/*
	for(int i = 0 ; i < width_dst * height_dst ; i++)
	   	if(isnan(scale_image[i])) print("YX_scale_gray(): isnan in scale_image\n");
	*/

	return 1;
}

int YX_scale_rgb(int width_src, int height_src, real *src_, int width_dst, int height_dst, real *dst_, real *buffer_)
{
	rgbReal *image = (rgbReal *) src_;
	rgbReal *scale_image = (rgbReal *) dst_;
	rgbReal *buffer = (rgbReal *) buffer_;
	
	float
		x_scale,
		x_scale_const,
		x_span,
		y_scale,
		y_scale_const,
		y_span;

	rgbReal	
		pixel,
		*scale_scanline,
		*scanline,
		*x_vector,
		*y_vector,
		zero;

	int	
		number_rows, y;

	float max_pixel = 255.0;

	register const rgbReal
		*p;

	register int
		i,
		x;

	register rgbReal
		*q;

	register rgbReal
		*s,
		*t;

	int
		next_column,
		next_row;

	//  
	x_vector = &buffer[0];

	//
	if(height_src != height_dst) scanline = &buffer[width_src];
   	else scanline = x_vector;

	//
	y_vector = &buffer[2*width_src];
	
	//
	scale_scanline = &buffer[3*width_src];

	/*
		Scale image
	*/
	number_rows=0;
	next_row=1;
	y_span=1.0;
        x_scale_const = (float) width_dst/width_src;
	y_scale_const = (float) height_dst/height_src;
	y_scale = y_scale_const;
	
	zero.r = 0.0;
	zero.g = 0.0;
	zero.b = 0.0;
	
	i=0;
	for (y=0; y < height_dst; y++)
	{
		q = &scale_image[y*width_dst];
		
		if (q == NULL)
			break;

    		if(height_dst == height_src)
      		{
        		/*
          			Read a new scanline.
        		*/
			p = &image[i*width_src];
			i++;
			
        		if (p == NULL)
          			break;
			
        		for (x=0; x < width_src; x++)
        		{
          			x_vector[x].r = p->r;
          			x_vector[x].g = p->g;
          			x_vector[x].b = p->b;
          			p++;
        		}
      		}
    		else
      		{
        		/*
          			Scale Y direction.
        		*/
        		while (y_scale < y_span)
        		{
          			if (next_row && (number_rows < height_src))
            			{
              				/*
                				Read a new scanline.
              				*/
					p = &image[i*width_src];
					i++;
				
              				if (p == NULL)
                				break;
				
              				for (x=0; x < width_src; x++)
              				{
          					x_vector[x].r = p->r;
          					x_vector[x].g = p->g;
          					x_vector[x].b = p->b;
                				p++;
              				}
              				number_rows++;
            			}
			
          			for (x=0; x < width_src; x++)
				{
            				y_vector[x].r += y_scale * x_vector[x].r;
            				y_vector[x].g += y_scale * x_vector[x].g;
            				y_vector[x].b += y_scale * x_vector[x].b;
				}

          			y_span-=y_scale;
          			y_scale = y_scale_const;
          			next_row=1;
        		}
		
        		if (next_row && (number_rows < height_src))
          		{
            			/*
              				Read a new scanline.
            			*/
				p = &image[i*width_src];
				i++;
			
            			if (p == NULL)
              				break;
			
            			for (x=0; x < width_src; x++)
            			{
          				x_vector[x].r = p->r;
          				x_vector[x].g = p->g;
          				x_vector[x].b = p->b;
              				p++;
            			}
            			number_rows++;
            			next_row=0;
          		}
		
        		s=scanline;
        		for (x=0; x < width_src; x++)
        		{
          			pixel.r = y_vector[x].r + y_span * x_vector[x].r;
          			pixel.g = y_vector[x].g + y_span * x_vector[x].g;
          			pixel.b = y_vector[x].b + y_span * x_vector[x].b;

          			s->r = pixel.r > max_pixel ? max_pixel : pixel.r;
          			s->g = pixel.g > max_pixel ? max_pixel : pixel.g;
          			s->b = pixel.b > max_pixel ? max_pixel : pixel.b;

          			s++;

          			y_vector[x].r = 0;
          			y_vector[x].g = 0;
          			y_vector[x].b = 0;
        		}
		
        		y_scale-=y_span;
        		if (y_scale <= 0)
          		{
            			y_scale = y_scale_const;
            			next_row=1;
          		}
        		y_span=1.0;
      		}
	
    		if (width_dst == width_src)
      		{
        		/*
          			Transfer scanline to scaled image.
        		*/
        		s=scanline;
        		for (x=0; x < width_dst; x++)
        		{
          			*q = *s;
          			q++;
          			s++;
        		}
      		}
    		else
      		{
        		/*
          			Scale X direction.
        		*/
        		pixel=zero;
        		next_column=0;
        		x_span=1.0;
        		s=scanline;
        		t=scale_scanline;
        		for (x=0; x < width_src; x++)
        		{
          			x_scale = x_scale_const;

          			while (x_scale >= x_span)
          			{
            				if (next_column)
              				{
                				pixel=zero;
                				t++;
              				}
            				pixel.r += x_span * s->r;
            				pixel.g += x_span * s->g;
            				pixel.b += x_span * s->b;
            				t->r = pixel.r > max_pixel ? max_pixel : pixel.r;
            				t->g = pixel.g > max_pixel ? max_pixel : pixel.g;
            				t->b = pixel.b > max_pixel ? max_pixel : pixel.b;
            				x_scale-=x_span;
            				x_span=1.0;
            				next_column=1;
          			}
        
				if (x_scale > 0)
          			{
            				if (next_column)
              				{
                				pixel=zero;
                				next_column=0;
                				t++;
              				}
            				pixel.r += x_scale * s->r;
            				pixel.g += x_scale * s->g;
            				pixel.b += x_scale * s->b;
            				x_span-=x_scale;
          			}
        			s++;
      			}
      
			if (x_span > 0)
        		{
          			s--;
            			pixel.r += x_span * s->r;
            			pixel.g += x_span * s->g;
            			pixel.b += x_span * s->b;
        		}
      
			if (!next_column && ((t-scale_scanline) < width_dst))
			{
            			t->r = pixel.r > max_pixel ? max_pixel : pixel.r;
            			t->g = pixel.g > max_pixel ? max_pixel : pixel.g;
            			t->b = pixel.b > max_pixel ? max_pixel : pixel.b;
			}

			/*
        			Transfer scanline to scaled image.
      			*/
      			t=scale_scanline;
      			for (x=0; x < width_dst; x++)
      			{
        			*q = *t;
        			q++;
        			t++;
      			}
    		}
	}

	return 1;
}


}

