#ifndef IPSHIFT_INC
#define IPSHIFT_INC

#include "ipCore.h"

namespace Torch {

/** This class is designed to shift the content of an image

    \begin{verbatim}

         +---+          +--------------+         +---+
         |XXX|	        |              |         |...|
	 |XXX|   ---->  |   ipShift    | ---->   |.XX|
         |XXX|          |              |         |.XX|
         +---+          +--------------+         +---+

	 image                                shifted image 
	 
    \end{verbatim}
    
    @author Sebastien Marcel (marcel@idiap.ch)
    @version 2.0
    \Date
    @since 2.0
*/
class ipShift : public ipCore
{
   	// width of the input image
	int width;

	// height of the input image
	int height;

public:
   	/// x shift parameter
	int x_shift;
	
	/// y shift parameter
	int y_shift;

	//-----
	
	/** constructor
	
	    @param width_ is the #width# of the input image
	    @param height_ is the #height# of the input image
	    @param coding_ is the #coding# format of the input image
	*/
    	ipShift(int width_, int height_, const char *coding_);

	///
	virtual ~ipShift();
	
	///
	virtual void process(Sequence *seq_in);
};

}

#endif
