#ifndef IPWEIGHEDGAUSSIAN_INC
#define IPWEIGHEDGAUSSIAN_INC

#include "ipCore.h"

namespace Torch {

/** This class is designed to apply a weighed gaussian smoothing filter

    \begin{verbatim}
    "Face Recognition under Varying Lighting conditions using Self Quotient Image"
    Wang, Li and Wang, 2004
    \end{verbatim}
    
    @author Guillaume Heusch (heusch@idiap.ch)
    @version 2.0
    \Date
    @since 2.0
*/
class ipWeighedGaussian : public ipCore
{
   	// width of the input image
	int width;

	// height of the input image
	int height;

	// size of the gaussian kernel
	int size;
	int halfsize;

	// the pre-computed kernels
	real *kernel;
	real *w_kernel;
	
public:
   	/// variance of the kernel
	real sigma; 

	//-----
	
	/** constructor
	    
	    @param size_ is the size of the kernel
	    @param width_ is the #width# of the input image
	    @param height_ is the #height# of the input image
	    @param coding_ is the #coding# format of the input image
	*/
	ipWeighedGaussian(int size_, int width_, int height_, const char *coding_);
	
	///
	virtual ~ipWeighedGaussian();

	///
	void process(Sequence *seq_in);	
};

}

#endif
