#ifndef NON_ORIENTED_PATTERN_FUSION_INC
#define NON_ORIENTED_PATTERN_FUSION_INC

#include "Object.h"
#include "PatternMerger.h"

namespace Torch {
	
// Internal structures for storing overlapping patterns
struct sSubTableOverlap
{
	int index;
	float surf;
	bool tag;
};

struct sTableOverlap
{
	int n_overlap;
	int n_real_overlap;
	int surf;

	sSubTableOverlap *overlap;
};

/** This class is designed to merge non-oriented patterns
	
    @author Sebastien Marcel (marcel@idiap.ch)
    @version 2.0
    \Date
    @since 1.0
*/
class NonOrientedPatternFusion : public Object
{
public:
	/// verbose merging
	bool verbose;

	/// max number of patterns
	int max_patterns;

	/// number of output patterns (merged)
	int n_patterns;

	/// output patterns (merged)
	Pattern **patterns;
	
private:

	// overlap table
	sTableOverlap *TableOverlap;

	//
	int n_merged_patterns;
	Pattern **merged_patterns;

	// the PatternMerger to use
	PatternMerger *merger;

	//
	int view;

	//
	int IsInTagOverlap(int index, int N, sSubTableOverlap *suboverlap);
	
	//
	int IsInOverlap(int index, int N, sSubTableOverlap *suboverlap);
	
	//
	bool doTableOverlap(int N, Pattern **patterns_, float surfoverlap, bool useInclusion = false);

	//
	bool tagTableOverlap(int N, Pattern **patterns_);

	//
	bool resolveTableOverlap(int N, Pattern **patterns_);

public:
	/** merge non-oriented patterns
	
	    @param merger_ is the PatternMerger to use to merge patterns
	    @param view_ is the view of patterns to process
	    @max_patterns_ is the maximum number of patterns to deal with
	*/
	NonOrientedPatternFusion(PatternMerger *merger_, int view_ = 0, int max_patterns_ = 1000);

	///
	void reset();

	/** merge patterns
	
	    @param N is the number of patterns to merge
	    @param patterns_ is the list of patterns to merge
	    @param surfoverlap is the minimum overlap surface to merge two patterns
	    @param threshold_overlap is the minimum confidence value of patterns to merge
	    @param computeActivation if is true computes the activation of merged patterns
	*/
	bool doOverlap(int N, Pattern **patterns_, float surfoverlap, float threshold_overlap, bool computeActivation = false);

	///
	void printTableOverlap(int N);

	///
	~NonOrientedPatternFusion();
};

}

#endif
