#ifndef PATTERN_MERGER_INC
#define PATTERN_MERGER_INC

#include "Pattern.h"

namespace Torch {

/** This class is designed to merge patterns
  
    This class cumulates patterns to merge using the method add().
    Then, the resulting merge pattern is given by the method merge().
    This class should be derived to implement a specific merging strategy.
    
    @author Sebastien Marcel (marcel@idiap.ch)
    @version 2.0
    \Date
    @since 1.0

    @see Pattern
*/
class PatternMerger : public Object
{
public:
	///
        bool verbose;

	/// number of patterns to merge
	int n_patterns;

	//---
	
	///
	PatternMerger();

	/// ready to cumulate patterns to merge
	virtual void reset() = 0;
	
	/// add a pattern to the list of patterns to merge
	virtual void add(Pattern *p_) = 0;
	
	/// merge the list of patterns into #p_#
	virtual bool merge(Pattern *p_) = 0;

	///
	virtual ~PatternMerger();
};

/** This class is designed to merge patterns using means rule
	
    @author Sebastien Marcel (marcel@idiap.ch)
    @version 2.0
    \Date
    @since 1.0
*/
class meanRectPatternMerger : public PatternMerger
{
public:
	/// mean x center
	real m_cx;

	/// mean y center
	real m_cy;
	
	/// mean width
	real m_w;

	/// mean height
	real m_h;

	/// mean scale
	real m_scale;
	
	/// mean view
	real m_view;
	
	/// mean confidence
	real m_confidence;

	/// max confidence
	real max_confidence;

	//---
	
	///
	meanRectPatternMerger();

	/// ready to cumulate patterns to merge
	virtual void reset();
	
	/// add a pattern to the list of patterns to merge
	virtual void add(Pattern *p_);
	
	/// merge the list of patterns into #p_#
	virtual bool merge(Pattern *p_);

	///
	virtual ~meanRectPatternMerger();
};

}

#endif

