#ifndef IP_SUBWINDOW_MLP_CASCADE__INC
#define IP_SUBWINDOW_MLP_CASCADE_INC

#include "ipSubWindow.h"

#include "ImageGray.h"

#include "ipHistoEqual.h"
#include "ipSmoothGaussian3.h"

#include "MyMLP.h"
#include "MyMeanVarNorm.h"

namespace Torch {

/** This class is designed to classify a subwindow using a cascade of MLP
 
    This class should be used with a PyramidWindowScan.
	
    @author Sebastien Marcel (marcel@idiap.ch)
    @version 2.0
    \Date
    @since 1.0
*/
class ipSWmlpCascade : public ipSubWindow
{
	// gray image for verbosing purpose
	ImageGray *grayimage;

public:
	/// pattern size
        int size_pattern;
 
	/// number of stages 
	int n_stages;

	/// MLP cascade
	Machine **mlps;

	/// threshold for each stage
	real *thresholds;

	/// the normalization
  	MyMeanVarNorm *mv_norm;

	/// the image normalization preprocessing
	ipCore *enhancing;

	/// the smoothing
	ipCore *smoothing;

	/// the sub_image to store the current input image to process
	Sequence *sub_image;

	//-----

	///
	ipSWmlpCascade(	int width_image_, int height_image_, 
			int width_pattern_, int height_pattern_, 
			const char *model_filename_, real threshold_,
			bool verbose_ = false);

	///
	virtual void process(Sequence *seq_in);

	///
	virtual ~ipSWmlpCascade();
};

}

#endif
