#include "ViscaSerial.h"

namespace Torch {

ViscaSerial::ViscaSerial(char *device_, char port_) : cameraControl(device_, port_)
{
}

ViscaSerial::~ViscaSerial()
{
}

bool ViscaSerial::init()
{
   	sCommandPacket sndCommand, revCommand;
  	int number_of_times=0;

	if(bVerbose == true)
		printf("ViscaSerial::init ...\n");

     	// Assign address to camera
  	do 
	{
    		number_of_times++;
		
    		if(assignAddress() < 0) return false;
  
    		// Espected answer with only one device connected
    		sndCommand.packet[0]=0x88; 
		sndCommand.packet[1]=0x30;
    		sndCommand.packet[2]=0x02; 
		sndCommand.packet[3]=0xff;
    		sndCommand.length=4;

    		// Wait for an answer for max 1 sec.
    		if(waitingForReply(1, &revCommand) == 0) return false;

    		// Was it the espected answer ?
  	} while(!isEqual(&sndCommand, &revCommand) && number_of_times < 10);

  	if(number_of_times==10)
	{
	   	printf("ViscaSerial::init Impossible to assignAddress after %d trials.\n", number_of_times);
		
		return false; 
	}
	
  	number_of_times = 0;
  	do 
	{   
    		number_of_times++;

    		// Clear the interface
    		if(clearInterface() < 0) return false;

    		// Wait for an answer for max 5 sec
    		if(waitingForReply(5, &revCommand) == 0) return false;
 
    		// Espected answer
  
    		sndCommand.packet[0]=0x88; 
		sndCommand.packet[1]=0x01;
    		sndCommand.packet[2]=0x00; 
		sndCommand.packet[3]=0x01;
    		sndCommand.packet[4]=0xff;
    		sndCommand.length=5;
  
    		// Was it the espected answer ?
  	} while(!isEqual(&sndCommand, &revCommand) && number_of_times < 10);

  	if(number_of_times==10) return false;

	return true;
}

int ViscaSerial::assignAddress() 
{
  	sCommandPacket sndCommand;

	if(bVerbose == true)
		printf("ViscaSerial::assignAddress\n");
	
  	// Camera and command id
  	sndCommand.packet[0]=0x88; 
	sndCommand.packet[1]=0x30;
  	sndCommand.packet[2]=0x01;
  	sndCommand.packet[3]=0xff;
  	sndCommand.length=4;

  	// Send the command to the camera
  	return(send(&sndCommand));
}

int ViscaSerial::clearInterface() 
{
	sCommandPacket sndCommand;

	if(bVerbose == true)
		printf("ViscaSerial::clearInterface\n");
	
  	// Camera and command id
  	sndCommand.packet[0]=0x88; 
	sndCommand.packet[1]=0x01;
  	sndCommand.packet[2]=0x00; 
	sndCommand.packet[3]=0x01;
  	sndCommand.packet[4]=0xff;
  	sndCommand.length=5;

  	// Send the command to the camera
  	return(send(&sndCommand));
}


bool ViscaSerial::IsAck(sCommandPacket *command)
{
     	return (command->length == 3 
	      && command->packet[0] == 0x90 
	      && command->packet[1] == 0x41 
	      && command->packet[2] == 0xff);
}

bool ViscaSerial::IsCpl(sCommandPacket *command)
{
     return (command->length == 3
	   && command->packet[0] == 0x90 
	   && command->packet[1] == 0x51 
	   && command->packet[2] == 0xff);
}

}

