#ifndef STUMP_TRAINER_INC
#define STUMP_TRAINER_INC

#undef HIST_PRECISION_
#ifndef HIST_PRECISION_
#define HIST_PRECISION_ 100
#endif

#include "Trainer.h"
#include "StumpMachine.h"

namespace Torch {

/** This class is designed to train a StumpMachine for a 2-class problem (positive patterns/negative patterns) using histograms

    @author Yann Rodriguez (yann.rodriguez@idiap.ch)
    @author Sebastien Marcel (marcel@idiap.ch)
    @version 2.0
    \Date
    @since 1.0
*/
class StumpTrainer : public Trainer
{
public:
   	///
	bool verbose;

   	/// the weak classifier to train
	StumpMachine *stump_machine;
	
	/// id of the best feature
	int bestFeature;

	/// threshold of the best features
	real bestThreshold;

	/// direction of the best feature
	int bestDirection;

	/// number of positive patterns
	int n_pos;

	/// histogram of positive patterns
	real histPos[HIST_PRECISION_];

	/// cumulative histogram of positive patterns
	real hCumulPos[HIST_PRECISION_];

	/// number of negative patterns
	int n_neg;

	/// histogram of negative patterns
	real histNeg[HIST_PRECISION_];
	
	/// cumulative histogram of negative patterns
	real hCumulNeg[HIST_PRECISION_];

	//
	real shift, sample;

	//
	real min_err;

	//
	real *features;

	//
	real *targets;

	//
	void computeHistogram(DataSet *data_, int feature_id);

	//
	void computeBestThreshold(int feature_id);

	///-----
	
	///
	StumpTrainer(StumpMachine *stump_machine_);

	///
	virtual void train(DataSet *data_, MeasurerList *measurers);

	///
	virtual ~StumpTrainer();
};

}

#endif
